require "import"
import "android.content.Intent"
import "android.content.pm.PackageManager"
import "android.speech.RecognizerIntent"
import "android.app.Activity"
import "android.widget.Toast"
import "android.speech.tts.TextToSpeech"
import "java.util.Locale"

local VOICE_RECOGNITION_REQUEST_CODE = 100
local tts

-- Initialize TextToSpeech engine
tts = TextToSpeech(activity, TextToSpeech.OnInitListener {
    onInit = function(status)
        if status == TextToSpeech.SUCCESS then
            tts.setLanguage(Locale.US)
        else
            showToast("TextToSpeech not available")
        end
    end
})

-- Launches speech recognition intent
function startSpeechRecognition()
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, Locale.getDefault())
    intent.putExtra(RecognizerIntent.EXTRA_PROMPT, "Say the name of the application")
    activity.startActivityForResult(intent, VOICE_RECOGNITION_REQUEST_CODE)
end

-- Process speech recognition result
function onActivityResult(requestCode, resultCode, data)
    if requestCode == VOICE_RECOGNITION_REQUEST_CODE then
        if resultCode == Activity.RESULT_OK then
            local matches = data.getStringArrayListExtra(RecognizerIntent.EXTRA_RESULTS)
            if matches.size() > 0 then
                local appName = matches.get(0)
                openAppByName(appName)
            end
        end
    end
end

-- Attempts to open an app by its name, speaks the process
function openAppByName(appName)
    local packageManager = activity.getPackageManager()
    local packageName = getPackageByName(appName, packageManager)
    
    if packageName then
        local intent = packageManager.getLaunchIntentForPackage(packageName)
        if intent then
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            activity.startActivity(intent)
            speakOut(appName .. " is now opening.")
        end
    else
        speakOut("App '" .. appName .. "' is,not found.")
    end
end

-- Speak out text using the TTS engine
function speakOut(text)
    if tts then
        tts.speak(text, TextToSpeech.QUEUE_FLUSH, nil, '')
    end
end

-- Show text as a toast
function showToast(text)
    Toast.makeText(activity, text, Toast.LENGTH_SHORT).show()
end

-- Get package name by app name (approximate match)
function getPackageByName(appName, packageManager)
    local packages = packageManager.getInstalledApplications(PackageManager.GET_META_DATA)

    for i = 0, packages.size() - 1 do
        local packageInfo = packages.get(i)
        local label = tostring(packageManager.getApplicationLabel(packageInfo))
        if label:lower():find(appName:lower()) then
            return packageInfo.packageName
        end
    end
    
    return nil
end

-- Clean up TTS resources
function onDestroy()
    if tts ~= nil then
        tts.stop()
        tts.shutdown()
        tts = nil
    end
end

-- Start speech recognition
startSpeechRecognition()
